'use strict';

exports.__esModule = true;

var _isNumber2 = require('lodash/isNumber');

var _isNumber3 = _interopRequireDefault(_isNumber2);

var _isString2 = require('lodash/isString');

var _isString3 = _interopRequireDefault(_isString2);

var _isNull2 = require('lodash/isNull');

var _isNull3 = _interopRequireDefault(_isNull2);

var _isUndefined2 = require('lodash/isUndefined');

var _isUndefined3 = _interopRequireDefault(_isUndefined2);

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _react = require('react');

var React = _interopRequireWildcard(_react);

var _domLib = require('dom-lib');

var _classnames = require('classnames');

var _classnames2 = _interopRequireDefault(_classnames);

var _utils = require('rsuite-utils/lib/utils');

var _utils2 = require('../utils');

var _stringToObject2 = require('../utils/stringToObject');

var _stringToObject3 = _interopRequireDefault(_stringToObject2);

var _DropdownMenuItem = require('../_picker/DropdownMenuItem');

var _DropdownMenuItem2 = _interopRequireDefault(_DropdownMenuItem);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defaults(obj, defaults) { var keys = Object.getOwnPropertyNames(defaults); for (var i = 0; i < keys.length; i++) { var key = keys[i]; var value = Object.getOwnPropertyDescriptor(defaults, key); if (value && value.configurable && obj[key] === undefined) { Object.defineProperty(obj, key, value); } } return obj; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : _defaults(subClass, superClass); }

var DropdownMenu = function (_React$Component) {
  _inherits(DropdownMenu, _React$Component);

  function DropdownMenu(props) {
    _classCallCheck(this, DropdownMenu);

    var _this = _possibleConstructorReturn(this, _React$Component.call(this, props));

    _initialiseProps.call(_this);

    var cascadeItems = props.cascadeItems,
        cascadePathItems = props.cascadePathItems;

    _this.state = {
      cascadeItems: cascadeItems,
      cascadePathItems: cascadePathItems
    };
    return _this;
  }

  DropdownMenu.prototype.componentDidMount = function componentDidMount() {
    this.scrollToActiveItemTop();
  };

  DropdownMenu.prototype.componentWillReceiveProps = function componentWillReceiveProps(nextProps) {
    var _this2 = this;

    var cascadeItems = nextProps.cascadeItems,
        cascadePathItems = nextProps.cascadePathItems;

    if (!(0, _utils.shallowEqualArray)(cascadeItems, this.props.cascadeItems)) {
      this.setState({ cascadeItems: cascadeItems }, function () {
        _this2.scrollToActiveItemTop();
      });
    }

    if (!(0, _utils.shallowEqualArray)(cascadePathItems, this.props.cascadePathItems)) {
      this.setState({ cascadePathItems: cascadePathItems }, function () {
        _this2.scrollToActiveItemTop();
      });
    }
  };

  DropdownMenu.prototype.setCascadeCache = function setCascadeCache(items, layer, node, isLeafNode, callback) {
    var _state = this.state,
        _state$cascadeItems = _state.cascadeItems,
        cascadeItems = _state$cascadeItems === undefined ? [] : _state$cascadeItems,
        cascadePathItems = _state.cascadePathItems;

    var nextItems = [];
    var nextPathItems = [];

    for (var i = 0; i < cascadeItems.length && i < layer; i += 1) {
      nextItems.push(cascadeItems[i]);
      if (i < layer - 1 && cascadePathItems) {
        nextPathItems.push(cascadePathItems[i]);
      }
    }

    nextPathItems.push(node);
    if (!isLeafNode) {
      nextItems.push(items);
    }

    this.setState({
      cascadeItems: nextItems,
      cascadePathItems: nextPathItems
    }, callback);
  };

  DropdownMenu.prototype.stringToObject = function stringToObject(value) {
    var _props = this.props,
        labelKey = _props.labelKey,
        valueKey = _props.valueKey;

    return (0, _stringToObject3.default)(value, labelKey, valueKey);
  };

  DropdownMenu.prototype.scrollToActiveItemTop = function scrollToActiveItemTop() {
    var _this3 = this;

    if (!this.menus) {
      return;
    }
    this.menus.forEach(function (menu) {
      if (!menu) {
        return;
      }

      var activeItem = menu.querySelector('.' + _this3.addPrefix('item-focus'));

      if (activeItem) {
        var position = (0, _domLib.getPosition)(activeItem, menu);
        (0, _domLib.scrollTop)(menu, position.top);
      }
    });
  };

  DropdownMenu.prototype.renderCascadeNode = function renderCascadeNode(node, index, layer, focus) {
    var _props2 = this.props,
        activeItemValue = _props2.activeItemValue,
        valueKey = _props2.valueKey,
        labelKey = _props2.labelKey,
        childrenKey = _props2.childrenKey,
        disabledItemValues = _props2.disabledItemValues,
        renderMenuItem = _props2.renderMenuItem;


    var children = node[childrenKey];
    var value = node[valueKey];
    var label = node[labelKey];

    var disabled = disabledItemValues.some(function (disabledValue) {
      return (0, _utils.shallowEqual)(disabledValue, value);
    });

    // Use `value` in keys when If `value` is string or number
    var onlyKey = (0, _isString3.default)(value) || (0, _isNumber3.default)(value) ? value : index;

    return React.createElement(
      _DropdownMenuItem2.default,
      {
        classPrefix: this.addPrefix('item'),
        getItemData: this.getItemData.bind(this, node),
        key: layer + '-' + onlyKey,
        disabled: disabled,
        active: !(0, _isUndefined3.default)(activeItemValue) && (0, _utils.shallowEqual)(activeItemValue, value),
        focus: focus,
        value: node,
        className: children ? this.addPrefix('has-children') : undefined,
        onSelect: this.handleSelect.bind(this, layer, index)
      },
      renderMenuItem ? renderMenuItem(label, node) : label,
      children ? React.createElement('span', { className: this.addPrefix('caret') }) : null
    );
  };

  DropdownMenu.prototype.renderCascade = function renderCascade() {
    var _this4 = this;

    var _state2 = this.state,
        _state2$cascadeItems = _state2.cascadeItems,
        cascadeItems = _state2$cascadeItems === undefined ? [] : _state2$cascadeItems,
        cascadePathItems = _state2.cascadePathItems;
    var _props3 = this.props,
        menuWidth = _props3.menuWidth,
        menuHeight = _props3.menuHeight,
        valueKey = _props3.valueKey,
        renderMenu = _props3.renderMenu;


    var styles = {
      width: cascadeItems.length * menuWidth
    };

    var cascadeNodes = cascadeItems.map(function (children, layer) {
      var onlyKey = layer + '_' + children.length;
      var menu = React.createElement(
        'ul',
        null,
        children.map(function (item, index) {
          return _this4.renderCascadeNode(item, index, layer, cascadePathItems[layer] && (0, _utils.shallowEqual)(cascadePathItems[layer][valueKey], item[valueKey]));
        })
      );

      var parentNode = cascadePathItems[layer - 1];
      var node = React.createElement(
        'div',
        {
          key: onlyKey,
          className: _this4.addPrefix('column'),
          ref: function ref(_ref) {
            _this4.menus[layer] = _ref;
          },
          style: {
            height: menuHeight,
            width: menuWidth
          }
        },
        renderMenu ? renderMenu(children, menu, parentNode) : menu
      );
      return node;
    });
    return React.createElement(
      'div',
      { style: styles },
      cascadeNodes
    );
  };

  DropdownMenu.prototype.render = function render() {
    var _this5 = this;

    var _props4 = this.props,
        className = _props4.className,
        rest = _objectWithoutProperties(_props4, ['className']);

    var classes = (0, _classnames2.default)(this.addPrefix('items'), className);
    var unhandled = (0, _utils2.getUnhandledProps)(DropdownMenu, rest);

    return React.createElement(
      'div',
      _extends({}, unhandled, {
        className: classes,
        ref: function ref(_ref2) {
          _this5.menuBodyContainer = _ref2;
        }
      }),
      this.renderCascade()
    );
  };

  return DropdownMenu;
}(React.Component);

DropdownMenu.defaultProps = {
  data: [],
  disabledItemValues: [],
  cascadeItems: [],
  cascadePathItems: [],
  menuWidth: 120,
  menuHeight: 200,
  childrenKey: 'children',
  valueKey: 'value',
  labelKey: 'label'
};
DropdownMenu.handledProps = ['activeItemValue', 'cascadeItems', 'cascadePathItems', 'childrenKey', 'className', 'classPrefix', 'data', 'disabledItemValues', 'labelKey', 'menuHeight', 'menuWidth', 'onSelect', 'renderMenu', 'renderMenuItem', 'valueKey'];
DropdownMenu.propTypes = {
  classPrefix: _propTypes2.default.string.isRequired,
  data: _propTypes2.default.arrayOf(_propTypes2.default.any).isRequired,
  disabledItemValues: _propTypes2.default.arrayOf(_propTypes2.default.any).isRequired,
  activeItemValue: _propTypes2.default.any,
  childrenKey: _propTypes2.default.string.isRequired,
  valueKey: _propTypes2.default.string.isRequired,
  labelKey: _propTypes2.default.string.isRequired,
  menuWidth: _propTypes2.default.number.isRequired,
  menuHeight: _propTypes2.default.number.isRequired,
  className: _propTypes2.default.string,
  renderMenuItem: _propTypes2.default.func,
  renderMenu: _propTypes2.default.func,
  onSelect: _propTypes2.default.func,
  cascadeItems: _propTypes2.default.arrayOf(_propTypes2.default.any).isRequired,
  cascadePathItems: _propTypes2.default.arrayOf(_propTypes2.default.any).isRequired
};

var _initialiseProps = function _initialiseProps() {
  var _this6 = this;

  this.menus = [];

  this.handleSelect = function (layer, index, node, event) {
    var _props5 = _this6.props,
        onSelect = _props5.onSelect,
        childrenKey = _props5.childrenKey;

    var children = node[childrenKey];
    var isLeafNode = (0, _isUndefined3.default)(children) || (0, _isNull3.default)(children);

    var callback = function callback() {
      var _state$cascadePathIte = _this6.state.cascadePathItems,
          cascadePathItems = _state$cascadePathIte === undefined ? [] : _state$cascadePathIte;

      onSelect && onSelect(node, cascadePathItems, isLeafNode, event);
    };

    var items = (children || []).map(function (item) {
      return _extends({}, _this6.stringToObject(item), {
        parent: node
      });
    });

    _this6.setCascadeCache(items, layer + 1, node, isLeafNode, callback);
  };

  this.getItemData = function (itemData) {
    return itemData;
  };

  this.addPrefix = function (name) {
    return (0, _utils2.prefix)(_this6.props.classPrefix)(name);
  };

  this.menuBodyContainer = null;
};

exports.default = DropdownMenu;