'use strict';

exports.__esModule = true;

var _react = require('react');

var React = _interopRequireWildcard(_react);

var _classnames = require('classnames');

var _classnames2 = _interopRequireDefault(_classnames);

var _domLib = require('dom-lib');

var _utils = require('rsuite-utils/lib/utils');

var _constants = require('../utils/constants');

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _defaults(obj, defaults) { var keys = Object.getOwnPropertyNames(defaults); for (var i = 0; i < keys.length; i++) { var key = keys[i]; var value = Object.getOwnPropertyDescriptor(defaults, key); if (value && value.configurable && obj[key] === undefined) { Object.defineProperty(obj, key, value); } } return obj; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : _defaults(subClass, superClass); }

var INITIAL_PADDING = 12;
var PADDING = 16;

var TreeCheckNode = function (_React$Component) {
  _inherits(TreeCheckNode, _React$Component);

  function TreeCheckNode() {
    var _temp, _this, _ret;

    _classCallCheck(this, TreeCheckNode);

    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return _ret = (_temp = (_this = _possibleConstructorReturn(this, _React$Component.call.apply(_React$Component, [this].concat(args))), _this), _this.handleTreeToggle = function (event) {
      var _this$props = _this.props,
          onTreeToggle = _this$props.onTreeToggle,
          layer = _this$props.layer,
          nodeData = _this$props.nodeData;

      onTreeToggle && onTreeToggle(nodeData, layer, event);
    }, _this.handleSelect = function (event) {
      var _this$props2 = _this.props,
          classPrefix = _this$props2.classPrefix,
          onTreeToggle = _this$props2.onTreeToggle,
          onSelect = _this$props2.onSelect,
          hasChildren = _this$props2.hasChildren,
          layer = _this$props2.layer,
          disabled = _this$props2.disabled,
          disabledCheckbox = _this$props2.disabledCheckbox,
          nodeData = _this$props2.nodeData,
          checkState = _this$props2.checkState;


      if (disabled || disabledCheckbox) {
        return;
      }

      // 如果点击的是展开 icon 就 return
      if (event.target instanceof HTMLElement) {
        if ((0, _domLib.hasClass)(event.target.parentNode, classPrefix + '-node-expand-icon-wrapper')) {
          return;
        }
      }

      var isChecked = false;
      if (checkState === _constants.CHECK_STATE.UNCHECK || checkState === _constants.CHECK_STATE.INDETERMINATE) {
        isChecked = true;
      }

      if (checkState === _constants.CHECK_STATE.CHECK) {
        isChecked = false;
      }
      nodeData.check = isChecked;
      onSelect && onSelect(nodeData, layer, event);
    }, _this.renderIcon = function () {
      var _this$props3 = _this.props,
          onRenderTreeIcon = _this$props3.onRenderTreeIcon,
          hasChildren = _this$props3.hasChildren,
          nodeData = _this$props3.nodeData,
          classPrefix = _this$props3.classPrefix;

      var expandIcon = React.createElement('i', { className: classPrefix + '-node-expand-icon icon' });
      if (typeof onRenderTreeIcon === 'function') {
        expandIcon = onRenderTreeIcon(nodeData);
      }
      return hasChildren ? React.createElement(
        'div',
        {
          role: 'button',
          tabIndex: '-1',
          'data-ref': nodeData.refKey,
          className: classPrefix + '-node-expand-icon-wrapper',
          onClick: _this.handleTreeToggle
        },
        expandIcon
      ) : null;
    }, _this.renderLabel = function () {
      var _this$props4 = _this.props,
          classPrefix = _this$props4.classPrefix,
          nodeData = _this$props4.nodeData,
          onRenderTreeNode = _this$props4.onRenderTreeNode,
          label = _this$props4.label,
          layer = _this$props4.layer,
          disabled = _this$props4.disabled,
          disabledCheckbox = _this$props4.disabledCheckbox;

      var addPrefix = (0, _utils.prefix)(classPrefix);
      var input = React.createElement(
        'span',
        { className: addPrefix('input-wrapper') },
        React.createElement('input', {
          className: addPrefix('input'),
          type: 'checkbox',
          disabled: disabled,
          onChange: _this.handleSelect
        }),
        React.createElement('span', { className: addPrefix('inner') })
      );
      var custom = typeof onRenderTreeNode === 'function' ? onRenderTreeNode(nodeData) : label;
      return React.createElement(
        'span',
        {
          role: 'button',
          tabIndex: '-1',
          className: addPrefix('checknode-label'),
          title: label,
          'data-layer': layer,
          'data-key': nodeData.refKey,
          onClick: _this.handleSelect
        },
        !disabledCheckbox ? input : null,
        custom
      );
    }, _temp), _possibleConstructorReturn(_this, _ret);
  }

  TreeCheckNode.prototype.shouldComponentUpdate = function shouldComponentUpdate(nextProps) {
    return !(0, _utils.shallowEqual)(this.props, nextProps);
  };

  /**
   * 展开收缩节点
   */


  TreeCheckNode.prototype.render = function render() {
    var _classNames;

    var _props = this.props,
        classPrefix = _props.classPrefix,
        visible = _props.visible,
        active = _props.active,
        layer = _props.layer,
        disabled = _props.disabled,
        disabledCheckbox = _props.disabledCheckbox,
        checkState = _props.checkState;


    var addPrefix = (0, _utils.prefix)(classPrefix + '-node');
    var classes = (0, _classnames2.default)(classPrefix + '-node', (_classNames = {
      'text-muted': disabled
    }, _classNames[addPrefix('indeterminate')] = checkState === _constants.CHECK_STATE.INDETERMINATE, _classNames[addPrefix('checked')] = checkState === _constants.CHECK_STATE.CHECK, _classNames[addPrefix('disabled')] = disabled, _classNames[addPrefix('active')] = active, _classNames[addPrefix('disabled-checkbox')] = disabledCheckbox, _classNames));

    var styles = { paddingLeft: layer * PADDING + INITIAL_PADDING };

    return visible ? React.createElement(
      'div',
      { style: styles, className: classes },
      this.renderIcon(),
      this.renderLabel()
    ) : null;
  };

  return TreeCheckNode;
}(React.Component);

TreeCheckNode.defaultProps = {
  visible: true
};
TreeCheckNode.handledProps = ['active', 'checkState', 'classPrefix', 'disabled', 'disabledCheckbox', 'hasChildren', 'label', 'layer', 'nodeData', 'onRenderTreeIcon', 'onRenderTreeNode', 'onSelect', 'onTreeToggle', 'value', 'visible'];
TreeCheckNode.propTypes = {
  classPrefix: _propTypes2.default.string.isRequired,
  visible: _propTypes2.default.bool,
  label: _propTypes2.default.any,
  value: _propTypes2.default.any,
  nodeData: _propTypes2.default.object.isRequired,
  active: _propTypes2.default.bool,
  checkState: _propTypes2.default.oneOfType([function () {
    return (typeof (_constants.CHECK_STATE.UNCHECK == null ? {} : _constants.CHECK_STATE.UNCHECK) === 'function' ? _propTypes2.default.instanceOf(_constants.CHECK_STATE.UNCHECK == null ? {} : _constants.CHECK_STATE.UNCHECK) : _propTypes2.default.any).apply(this, arguments);
  }, function () {
    return (typeof (_constants.CHECK_STATE.INDETERMINATE == null ? {} : _constants.CHECK_STATE.INDETERMINATE) === 'function' ? _propTypes2.default.instanceOf(_constants.CHECK_STATE.INDETERMINATE == null ? {} : _constants.CHECK_STATE.INDETERMINATE) : _propTypes2.default.any).apply(this, arguments);
  }, function () {
    return (typeof (_constants.CHECK_STATE.CHECK == null ? {} : _constants.CHECK_STATE.CHECK) === 'function' ? _propTypes2.default.instanceOf(_constants.CHECK_STATE.CHECK == null ? {} : _constants.CHECK_STATE.CHECK) : _propTypes2.default.any).apply(this, arguments);
  }]),
  hasChildren: _propTypes2.default.bool,
  disabled: _propTypes2.default.bool,
  disabledCheckbox: _propTypes2.default.bool.isRequired,
  layer: _propTypes2.default.number.isRequired,
  onTreeToggle: _propTypes2.default.func,
  onSelect: _propTypes2.default.func,
  onRenderTreeIcon: _propTypes2.default.func,
  onRenderTreeNode: _propTypes2.default.func
};
exports.default = TreeCheckNode;