'use strict';

exports.__esModule = true;

var _isArray2 = require('lodash/isArray');

var _isArray3 = _interopRequireDefault(_isArray2);

var _isNumber2 = require('lodash/isNumber');

var _isNumber3 = _interopRequireDefault(_isNumber2);

var _isString2 = require('lodash/isString');

var _isString3 = _interopRequireDefault(_isString2);

var _isUndefined2 = require('lodash/isUndefined');

var _isUndefined3 = _interopRequireDefault(_isUndefined2);

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _react = require('react');

var React = _interopRequireWildcard(_react);

var _domLib = require('dom-lib');

var _classnames = require('classnames');

var _classnames2 = _interopRequireDefault(_classnames);

var _utils = require('rsuite-utils/lib/utils');

var _utils2 = require('../utils');

var _DropdownMenuGroup = require('./DropdownMenuGroup');

var _DropdownMenuGroup2 = _interopRequireDefault(_DropdownMenuGroup);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defaults(obj, defaults) { var keys = Object.getOwnPropertyNames(defaults); for (var i = 0; i < keys.length; i++) { var key = keys[i]; var value = Object.getOwnPropertyDescriptor(defaults, key); if (value && value.configurable && obj[key] === undefined) { Object.defineProperty(obj, key, value); } } return obj; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : _defaults(subClass, superClass); }

var DropdownMenu = function (_React$Component) {
  _inherits(DropdownMenu, _React$Component);

  function DropdownMenu() {
    var _temp, _this, _ret;

    _classCallCheck(this, DropdownMenu);

    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return _ret = (_temp = (_this = _possibleConstructorReturn(this, _React$Component.call.apply(_React$Component, [this].concat(args))), _this), _this.menuItems = {}, _this.menuBodyContainer = {}, _this.addPrefix = function (name) {
      return (0, _utils2.prefix)(_this.props.classPrefix)(name);
    }, _this.handleSelect = function (item, value, event, checked) {
      var onSelect = _this.props.onSelect;

      onSelect && onSelect(value, item, event, checked);
    }, _this.bindMenuItems = function (disabled, key, ref) {
      if (ref && !disabled) {
        _this.menuItems[key] = ref;
      }
    }, _this.getItemData = function (itemData) {
      return itemData;
    }, _temp), _possibleConstructorReturn(_this, _ret);
  }

  /**
   * Note that `handledProps` are generated automatically during
   * build with `babel-plugin-transform-react-flow-handled-props`
   */
  DropdownMenu.prototype.componentDidMount = function componentDidMount() {
    this.updateScrollPoistion();
  };

  DropdownMenu.prototype.componentDidUpdate = function componentDidUpdate(prevProps) {
    if (!(0, _utils.shallowEqual)(prevProps.focusItemValue, this.props.focusItemValue)) {
      this.updateScrollPoistion();
    }
  };

  DropdownMenu.prototype.updateScrollPoistion = function updateScrollPoistion() {
    var activeItem = this.menuBodyContainer.querySelector('.' + this.addPrefix('item-focus'));
    if (!activeItem) {
      return;
    }
    var position = (0, _domLib.getPosition)(activeItem, this.menuBodyContainer);
    var sTop = (0, _domLib.scrollTop)(this.menuBodyContainer);
    var sHeight = (0, _domLib.getHeight)(this.menuBodyContainer);
    if (sTop > position.top) {
      (0, _domLib.scrollTop)(this.menuBodyContainer, Math.max(0, position.top - 20));
    } else if (position.top > sTop + sHeight) {
      (0, _domLib.scrollTop)(this.menuBodyContainer, Math.max(0, position.top - sHeight + 32));
    }
  };

  DropdownMenu.prototype.renderItems = function renderItems() {
    var _this2 = this;

    var _props = this.props,
        activeItemValues = _props.activeItemValues,
        focusItemValue = _props.focusItemValue,
        valueKey = _props.valueKey,
        labelKey = _props.labelKey,
        data = _props.data,
        renderMenuItem = _props.renderMenuItem,
        renderMenuGroup = _props.renderMenuGroup,
        onGroupTitleClick = _props.onGroupTitleClick,
        disabledItemValues = _props.disabledItemValues,
        group = _props.group,
        dropdownMenuItemClassPrefix = _props.dropdownMenuItemClassPrefix,
        DropdownMenuItem = _props.dropdownMenuItemComponentClass;


    this.menuItems = {};

    var createMenuItems = function createMenuItems() {
      var items = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];
      var groupId = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
      return items.map(function (item, index) {
        var value = item[valueKey];
        var label = item[labelKey];

        if ((0, _isUndefined3.default)(label) && (0, _isUndefined3.default)(item.groupTitle)) {
          throw Error('labelKey "' + labelKey + '" is not defined in "data" : ' + index);
        }

        // Use `value` in keys when If `value` is string or number
        var onlyKey = (0, _isString3.default)(value) || (0, _isNumber3.default)(value) ? value : index;

        /**
         * Render <DropdownMenuGroup>
         * when if `group` is enabled and `itme.children` is array
         */
        if (group && (0, _isArray3.default)(item.children)) {
          return React.createElement(
            _DropdownMenuGroup2.default,
            {
              classPrefix: _this2.addPrefix('group'),
              key: onlyKey,
              title: renderMenuGroup ? renderMenuGroup(item.groupTitle, item) : item.groupTitle,
              onClick: onGroupTitleClick
            },
            createMenuItems(item.children, onlyKey)
          );
        } else if ((0, _isUndefined3.default)(value) && !(0, _isArray3.default)(item.children)) {
          throw Error('valueKey "' + valueKey + '" is not defined in "data" : ' + index + ' ');
        }

        var disabled = disabledItemValues.some(function (disabledValue) {
          return (0, _utils.shallowEqual)(disabledValue, value);
        });

        return React.createElement(
          DropdownMenuItem,
          {
            classPrefix: dropdownMenuItemClassPrefix,
            getItemData: _this2.getItemData.bind(_this2, item),
            key: groupId + '-' + onlyKey,
            disabled: disabled,
            active: !(0, _isUndefined3.default)(activeItemValues) && activeItemValues.some(function (v) {
              return (0, _utils.shallowEqual)(v, value);
            }),
            focus: !(0, _isUndefined3.default)(focusItemValue) && (0, _utils.shallowEqual)(focusItemValue, value),
            value: value,
            ref: _this2.bindMenuItems.bind(_this2, disabled, groupId + '-' + onlyKey),
            onSelect: _this2.handleSelect.bind(_this2, item)
          },
          renderMenuItem ? renderMenuItem(label, item) : label
        );
      });
    };

    createMenuItems.handledProps = [];
    return createMenuItems(data);
  };

  DropdownMenu.prototype.render = function render() {
    var _this3 = this;

    var _props2 = this.props,
        maxHeight = _props2.maxHeight,
        className = _props2.className,
        style = _props2.style,
        classPrefix = _props2.classPrefix,
        rest = _objectWithoutProperties(_props2, ['maxHeight', 'className', 'style', 'classPrefix']);

    var classes = (0, _classnames2.default)(this.addPrefix('items'), className);
    var unhandled = (0, _utils2.getUnhandledProps)(DropdownMenu, rest);

    return React.createElement(
      'div',
      _extends({}, unhandled, {
        className: classes,
        ref: function ref(_ref) {
          _this3.menuBodyContainer = _ref;
        },
        style: _extends({}, style, {
          maxHeight: maxHeight
        })
      }),
      React.createElement(
        'ul',
        null,
        this.renderItems()
      )
    );
  };

  return DropdownMenu;
}(React.Component);

DropdownMenu.defaultProps = {
  data: [],
  activeItemValues: [],
  disabledItemValues: [],
  maxHeight: 320,
  valueKey: 'value',
  labelKey: 'label'
};
DropdownMenu.handledProps = ['activeItemValues', 'className', 'classPrefix', 'data', 'disabledItemValues', 'dropdownMenuItemClassPrefix', 'dropdownMenuItemComponentClass', 'focusItemValue', 'group', 'labelKey', 'maxHeight', 'onGroupTitleClick', 'onSelect', 'renderMenuGroup', 'renderMenuItem', 'style', 'valueKey'];
DropdownMenu.propTypes = {
  classPrefix: _propTypes2.default.string.isRequired,
  data: _propTypes2.default.arrayOf(_propTypes2.default.any),
  group: _propTypes2.default.bool,
  disabledItemValues: _propTypes2.default.arrayOf(_propTypes2.default.any).isRequired,
  activeItemValues: _propTypes2.default.arrayOf(_propTypes2.default.any).isRequired,
  focusItemValue: _propTypes2.default.any,
  maxHeight: _propTypes2.default.number,
  valueKey: _propTypes2.default.string.isRequired,
  labelKey: _propTypes2.default.string.isRequired,
  className: _propTypes2.default.string,
  style: _propTypes2.default.object,
  renderMenuItem: _propTypes2.default.func,
  renderMenuGroup: _propTypes2.default.func,
  onSelect: _propTypes2.default.func,
  onGroupTitleClick: _propTypes2.default.func,
  dropdownMenuItemComponentClass: function dropdownMenuItemComponentClass() {
    return (typeof (React.ElementType == null ? {} : React.ElementType) === 'function' ? _propTypes2.default.instanceOf(React.ElementType == null ? {} : React.ElementType).isRequired : _propTypes2.default.any.isRequired).apply(this, arguments);
  },
  dropdownMenuItemClassPrefix: _propTypes2.default.string
};
exports.default = DropdownMenu;